var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { AbstractPowerSyncDatabase, SqliteBucketStorage } from '@journeyapps/powersync-sdk-common';
import { WebRemote } from './sync/WebRemote';
import { SharedWebStreamingSyncImplementation } from './sync/SharedWebStreamingSyncImplementation';
import { SSRStreamingSyncImplementation } from './sync/SSRWebStreamingSyncImplementation';
import { WebStreamingSyncImplementation } from './sync/WebStreamingSyncImplementation';
export class PowerSyncDatabase extends AbstractPowerSyncDatabase {
    constructor(options) {
        super(options);
        this.options = options;
    }
    _initialize() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    generateBucketStorageAdapter() {
        return new SqliteBucketStorage(this.database, AbstractPowerSyncDatabase.transactionMutex);
    }
    generateSyncStreamImplementation(connector) {
        const remote = new WebRemote(connector);
        const syncOptions = Object.assign(Object.assign({}, this.options), { adapter: this.bucketStorageAdapter, remote, uploadCrud: () => __awaiter(this, void 0, void 0, function* () {
                yield this.waitForReady();
                yield connector.uploadData(this);
            }), workerIdentifier: this.options.database.name });
        const { flags } = this.options;
        switch (true) {
            case flags === null || flags === void 0 ? void 0 : flags.ssrMode:
                return new SSRStreamingSyncImplementation(syncOptions);
            case flags === null || flags === void 0 ? void 0 : flags.enableMultiTabs:
                return new SharedWebStreamingSyncImplementation(syncOptions);
            default:
                return new WebStreamingSyncImplementation(syncOptions);
        }
    }
}
//# sourceMappingURL=PowerSyncDatabase.js.map